use crate::{middleware::AuthenticatedAdminMiddleware, AppState};
use actix_multipart::Multipart;
use actix_web::{
    post,
    web::{block, Data},
    HttpRequest, HttpResponse, Responder,
};
use futures::StreamExt;
use std::{
    fs::{remove_file, File},
    io::Write,
    path::Path,
    sync::Arc,
};

#[post("/post-file")]
pub async fn post_file(
    app_state: Data<AppState>,
    mut payload: Multipart,
    middleware: Data<AuthenticatedAdminMiddleware<'_>>,
    req: HttpRequest,
) -> impl Responder {
    if middleware.exec(&app_state, &req, None).await.is_err() {
        return HttpResponse::Unauthorized().finish();
    }

    while let Some(item) = payload.next().await {
        match item {
            Ok(mut field) => {
                // Field in turn is stream of *Bytes* object
                // A multipart/form-data stream has to contain `content_disposition`
                let content_disposition = field
                    .content_disposition()
                    .expect("Missing Content Disposition header");

                let filename = content_disposition.get_filename().expect("Missin filename");

                let filepath = Arc::new(Path::new(format!("./tmp/{filename}")));

                // File::create is blocking operation, use threadpool
                let mut f = block(|| File::create(*filepath.clone())).await;

                if f.is_err() {
                    return HttpResponse::InternalServerError()
                        .body(format!("Error creating file {:?}", f));
                }

                let f = f.unwrap();

                let mut error = None;

                // Field in turn is stream of *Bytes* object
                'chunks: while let Some(chunk) = field.next().await {
                    match chunk {
                        Ok(chunk) => {
                            // filesystem operations are blocking, we have to use threadpool
                            if block(move || f.write_all(&chunk).map(|_| f)).await.is_err() {
                                error = Some("Error writing chunk".to_string());
                                break 'chunks;
                            }
                        }
                        Err(e) => {
                            error = format!("Error writing file {} : {:?}", filename, e);
                            break 'chunks;
                        }
                    }
                }

                if let Some(err) = error {
                    block(|| remove_file(*filepath.clone())).await.unwrap();
                    return HttpResponse::InternalServerError().body(err);
                }
            }
            Err(e) => {
                return HttpResponse::InternalServerError().body(format!("FIELD ERR {:?}", e))
            }
        }
    }

    HttpResponse::Ok().body("File was successfully uploaded")
}

// use futures::TryStreamExt;
// use std::io::Write;

// pub async fn save_file(mut payload: Multipart) -> impl Responder {
//     println!("SAVE FILE");
//     // iterate over multipart stream
//     while let Some(mut field) = payload.try_next().await.unwrap() {
//         // A multipart/form-data stream has to contain `content_disposition`
//         let content_disposition = field.content_disposition().unwrap();
//         let filename = content_disposition.get_filename().unwrap();
//         let filepath = format!("./tmp/{filename}");

//         // File::create is blocking operation, use threadpool
//         let mut f = actix_web::web::block(|| std::fs::File::create(filepath))
//             .await
//             .unwrap();

//         // Field in turn is stream of *Bytes* object
//         while let Some(chunk) = field.try_next().await.unwrap() {
//             // filesystem operations are blocking, we have to use threadpool
//             f = actix_web::web::block(move || f.write_all(&chunk).map(|_| f))
//                 .await
//                 .unwrap();
//         }
//     }

//     HttpResponse::Ok().body("sucess")
// }
